#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Chars.h"
#include "Print.h"
#include "Sprite.h"
#include "ScanKeys.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    // ClearScreen();

    PrintS(Vram + 24, "SCORE");
    PrintS(Vram + VramWidth * 3 + 24, "HI-SCORE");
    PrintS(Vram + VramWidth * 6 + 24, "STAGE");
    PrintByteNumber2(Vram + VramWidth * 6 + 30, CurrentStage + 1);
    PrintScore();
    PrintRemain();
    {
        ptr<byte> pVram;
        pVram= Vram + VramWidth * 12 + 24;
        repeat(12) {
            repeat(8) {
                *pVram = Char_Map; ++pVram;
            }
            pVram += VramWidth - 8;
        }
    }
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramWidth + 26, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramWidth * 4 + 26, HiScore);
    PrintC(pVram, '0');
}

void PrintRemain()
{
    ptr<byte> pVram;
    byte i;
    pVram= Vram + VramWidth * 9 + 24;
    if (RemainFighterCount < 5) {
        i = 1;
        while (i < RemainFighterCount) {
            VPut2(pVram, Char_Remain);
            pVram += 2;
            ++i;
        }
    }
    else {
        VPut2(pVram, Char_Remain);
        pVram += 2;
        VErase2(pVram);
        pVram += 2;
        PrintByteNumber2(Vram + VramWidth * 10 + 26, RemainFighterCount - 1);
        i = 3;
    }
    while (i < 4) {
        VErase2(pVram);
        pVram += 2;
        ++i;
    }
}


static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramWidth * 11 + 7;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramWidth - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramWidth * 12 + 8, "GAME OVER");
}

static const byte[] TitleBytes = {
    //	R
    0x27, 0x22, 0x20, 0x26, 
    0x27, 0x1e, 0x23, 0x2a, 
    0x27, 0x22, 0x2a, 0x1d, 
    0x1f, 0x1c, 0x1f, 0x20, 
    //	U
    0x27, 0x1e, 0x1b, 0x2a, 
    0x27, 0x1e, 0x1b, 0x2a, 
    0x27, 0x1e, 0x1b, 0x2a, 
    0x1b, 0x20, 0x20, 0x1c, 
    //	P
    0x27, 0x22, 0x20, 0x26, 
    0x27, 0x1e, 0x1b, 0x2a, 
    0x27, 0x22, 0x20, 0x1c, 
    0x1f, 0x1c, 0x1b, 0x1b, 
    //	T
    0x1f, 0x28, 0x22, 0x1c, 
    0x1b, 0x27, 0x1e, 0x1b, 
    0x1b, 0x27, 0x1e, 0x1b, 
    0x1b, 0x1f, 0x1c, 0x1b, 
    //	U
    0x27, 0x1e, 0x1b, 0x2a, 
    0x27, 0x1e, 0x1b, 0x2a, 
    0x27, 0x1e, 0x1b, 0x2a, 
    0x1b, 0x20, 0x20, 0x1c, 
    //	S
    0x23, 0x22, 0x20, 0x26, 
    0x1f, 0x26, 0x25, 0x1d, 
    0x23, 0x1d, 0x1b, 0x2a, 
    0x1b, 0x20, 0x20, 0x1c, 
};

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    PrintStatus();

    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramWidth * 8;
        p = TitleBytes;
        repeat (6) {
            VPut4(pVram, p);
            pVram += 4;
            p += 16;
        }
    }
    PrintS(Vram + VramWidth * 17 + 3, "PUSH TRIGGER BUTTON");
    // {
    //     ptr<byte> pVram;
    //     byte c;
    //     pVram = Vram;
    //     c = 0;
    //     repeat(0) {
    //         *pVram = c; ++pVram;
    //         ++c;
    //         if ((c & 15) == 0) {
    //             pVram += VramWidth - 16;
    //         }
    //     }
    // }
}

void PrintRader(byte x, byte y, byte c)
{
    word w;
    ptr<byte> pVram;
    x = (x - 8) >> 4;
    y = (y - 8) >> 4;
    w = y;
    w <<= 5;
    pVram = Vram + VramWidth * 12 + 24 + w + x;
    *pVram = c;
}